/*
 * JBoss, Home of Professional Open Source
 * Copyright 2009 Red Hat Inc. and/or its affiliates and other
 * contributors as indicated by the @author tags. All rights reserved.
 * See the copyright.txt in the distribution for a full listing of
 * individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.infinispan.remoting.jgroups;

import org.easymock.classextension.EasyMock;
import org.infinispan.config.GlobalConfiguration;
import org.infinispan.factories.GlobalComponentRegistry;
import org.infinispan.manager.CacheContainer;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.remoting.transport.Transport;
import org.infinispan.remoting.transport.jgroups.JGroupsChannelLookup;
import org.infinispan.remoting.transport.jgroups.JGroupsTransport;
import org.infinispan.test.AbstractInfinispanTest;
import org.infinispan.test.TestingUtil;
import org.infinispan.test.fwk.TestCacheManagerFactory;
import org.jgroups.Address;
import org.jgroups.Channel;
import org.jgroups.JChannel;
import org.jgroups.protocols.UDP;
import org.jgroups.stack.ProtocolStack;
import org.testng.annotations.Test;

import java.util.Properties;

@Test(testName = "remoting.jgroups.ChannelLookupTest", groups = "functional")
public class ChannelLookupTest extends AbstractInfinispanTest {
    public void channelLookupTest() {
        EmbeddedCacheManager cm = null;
        try {
            GlobalConfiguration gc = GlobalConfiguration.getClusteredDefault();
            Properties p = new Properties();
            p.setProperty("channelLookup", DummyLookup.class.getName());
            gc.setTransportProperties(p);
            cm = TestCacheManagerFactory.createCacheManager(gc);
            cm.start();
            cm.getCache();
            GlobalComponentRegistry gcr = TestingUtil.extractGlobalComponentRegistry(cm);
            Transport t = gcr.getComponent(Transport.class);
            assert t != null;
            assert t instanceof JGroupsTransport;
            assert !(((JGroupsTransport) t).getChannel() instanceof JChannel);
        } finally {
            TestingUtil.killCacheManagers(cm);
        }
    }

    public static class DummyLookup implements JGroupsChannelLookup {
        Channel mockChannel;
        Address a = EasyMock.createNiceMock(Address.class);
        ProtocolStack ps = EasyMock.createNiceMock(ProtocolStack.class);

        public DummyLookup() {
            mockChannel = EasyMock.createNiceMock(Channel.class);
            EasyMock.expect(mockChannel.getAddress()).andReturn(a);
            EasyMock.expect(mockChannel.getProtocolStack()).andReturn(ps);
            EasyMock.expect(ps.getTransport()).andReturn(new UDP());
            EasyMock.replay(mockChannel, a, ps);           
        }

        public Channel getJGroupsChannel(Properties p) {
            return mockChannel;
        }

        public boolean shouldStartAndConnect() {
            return false;
        }

        public boolean shouldStopAndDisconnect() {
            return false;
        }
    }
}
